/*
 * SPDX-FileCopyrightText: 2023 jordi fita mas <jfita@peritasoft.com>
 * SPDX-License-Identifier: AGPL-3.0-only
 */

package types

import (
	"context"
	"net/http"

	"dev.tandem.ws/tandem/camper/pkg/auth"
	"dev.tandem.ws/tandem/camper/pkg/database"
	"dev.tandem.ws/tandem/camper/pkg/form"
	httplib "dev.tandem.ws/tandem/camper/pkg/http"
	"dev.tandem.ws/tandem/camper/pkg/locale"
	"dev.tandem.ws/tandem/camper/pkg/template"
)

type typeL10nForm struct {
	Locale      *locale.Locale
	Slug        string
	Name        *form.L10nInput
	Description *form.L10nInput
}

func newTypeL10nForm(f *typeForm, loc *locale.Locale) *typeL10nForm {
	return &typeL10nForm{
		Locale:      loc,
		Slug:        f.Slug,
		Name:        f.Name.L10nInput(),
		Description: f.Description.L10nInput(),
	}
}

func (l10n *typeL10nForm) FillFromDatabase(ctx context.Context, conn *database.Conn) error {
	row := conn.QueryRow(ctx, `
		select coalesce(i18n.name, '') as l10n_name
		     , coalesce(i18n.description, '') as l10n_description
		from campsite_type
		    left join campsite_type_i18n as i18n on campsite_type.campsite_type_id = i18n.campsite_type_id and i18n.lang_tag = $1
		where slug = $2
	`, l10n.Locale.Language, l10n.Slug)
	return row.Scan(&l10n.Name.Val, &l10n.Description.Val)
}

func (l10n *typeL10nForm) MustRender(w http.ResponseWriter, r *http.Request, user *auth.User, company *auth.Company) {
	template.MustRenderAdmin(w, r, user, company, "campsite/type/l10n.gohtml", l10n)
}

func editTypeL10n(w http.ResponseWriter, r *http.Request, user *auth.User, company *auth.Company, conn *database.Conn, l10n *typeL10nForm) {
	if err := l10n.Parse(r); err != nil {
		http.Error(w, err.Error(), http.StatusBadRequest)
		return
	}
	if err := user.VerifyCSRFToken(r); err != nil {
		http.Error(w, err.Error(), http.StatusForbidden)
		return
	}
	if !l10n.Valid(user.Locale) {
		if !httplib.IsHTMxRequest(r) {
			w.WriteHeader(http.StatusUnprocessableEntity)
		}
		l10n.MustRender(w, r, user, company)
		return
	}
	conn.MustExec(r.Context(), "select translate_campsite_type($1, $2, $3, $4)", l10n.Slug, l10n.Locale.Language, l10n.Name, l10n.Description)
	httplib.Redirect(w, r, "/admin/campsites/types", http.StatusSeeOther)
}

func (l10n *typeL10nForm) Parse(r *http.Request) error {
	if err := r.ParseForm(); err != nil {
		return err
	}
	l10n.Name.FillValue(r)
	l10n.Description.FillValue(r)
	return nil
}

func (l10n *typeL10nForm) Valid(l *locale.Locale) bool {
	v := form.NewValidator(l)
	v.CheckRequired(&l10n.Name.Input, l.GettextNoop("Name can not be empty."))
	return v.AllOK
}
